#ifndef __DATETIME_H__
#define __DATETIME_H__

#include <string>

// Any comments to christian.reinbothe@t-online.de!

class Date
{
   public:

      enum Months
      {
         JANUARY = 1,
         FEBRUARY = 2,
         MARCH = 3,
         APRIL = 4,
         MAY = 5,
         JUNE = 6,
         JULY = 7,
         AUGUST = 8,
         SEPTEMBER = 9,
         OCTOBER = 10,
         NOVEMBER = 11,
         DECEMBER = 12
      };

      enum Days
      {
         SUNDAY = 0,
         MONDAY = 1,
         TUESDAY = 2,
         WEDNESDAY = 3,
         THURSDAY = 4,
         FRIDAY = 5,
         SATURDAY = 6
      };

      static const int N_WEEK = 7;

      static const int N_MONTH = 12;

      static const int N_JANUARY = 31;
      static const int N_FEBRUARY = 28;
      static const int N_FEBRUARY_LEAP_YEAR = 29;
      static const int N_MARCH = 31;
      static const int N_APRIL = 30;
      static const int N_MAY = 31;
      static const int N_JUNE = 30;
      static const int N_JULY = 31;
      static const int N_AUGUST = 31;
      static const int N_SEPTEMBER = 30;
      static const int N_OCTOBER = 31;
      static const int N_NOVEMBER = 30;
      static const int N_DECEMBER = 31;
 
      static const int LEAP_YEAR_400 = 400;
      static const int LEAP_YEAR_100 = 100;
      static const int LEAP_YEAR_4 = 4;

      int year;
      int month;
      int day;
      int day_of_the_week;

      Date();
      Date(const Date& D);
      Date(int ye, int mo, int da);
      Date(const char *S);
      Date(const std::string S);
      ~Date();

      std::string to_string_US();
      std::string to_string_DE();

      static Date EasternJulianic(int ye);
      static Date EasternGregorianic(int ye);

      static int DaysInMonth(int ye, int mo);

      static bool IsLeapYear(int ye);

      bool IsDate();
      static bool IsDate(int ye, int mo, int da);
      static bool IsDate(const char *S);
      static bool IsDateUS(const char *S);
      static bool IsDateDE(const char *S);
      static bool IsDate(const std::string S);
      static bool IsDateUS(const std::string S);
      static bool IsDateDE(const std::string S);
      
      static int Compute_dotw(int ye, int mo, int da);

      Date& operator =(const Date& D);

      Date& operator ++();
      Date& operator --();

      Date operator ++(int i);
      Date operator --(int i);

      bool operator ==(const Date& D);
      bool operator !=(const Date& D);

      bool operator <(const Date& D);
      bool operator >(const Date& D);

      bool operator <=(const Date& D);
      bool operator >=(const Date& D);

      int operator -(const Date& D);

      Date operator +(int ndays);
      Date operator -(int ndays);
};

class Time
{
   public:

      static const int N_HOUR = 24;
      static const int N_MINUTE = 60;
      static const int N_SECOND = 60;

      int hour;
      int minute;
      int second;
   
      Time();
      Time(const Time& T);
      Time(int ho, int mi, int se);
      Time(const char *S);
      Time(const std::string S);
      ~Time();

      std::string to_string();

      int to_seconds();
      double to_minutes();
      double to_hours();

      bool IsTime();
      static bool IsTime(int ho, int mi, int se);
      static bool IsTime(const char *S);
      static bool IsTime(const std::string S);

      Time& operator =(const Time& T);

      bool operator ==(const Time& T);
      bool operator !=(const Time& T);

      bool operator <(const Time& T);
      bool operator >(const Time& T);

      bool operator <=(const Time& T);
      bool operator >=(const Time& T);
};

class DateTime
{
   public:

      Date D;
      Time T;

      DateTime();
      DateTime(const Date& D, const Time& T);
      DateTime(const DateTime& DT);
      DateTime(int year, int month, int day, int hour, int minute, int second);
      DateTime(const char *S);
      DateTime(const std::string S);
      ~DateTime();

      std::string to_string_US();
      std::string to_string_DE();

      bool IsDateTime();
      static bool IsDateTime(int year, int month, int day, int hour, int minute, int second);
      static bool IsDateTime(const char *S);
      static bool IsDateTime(const std::string S);

      DateTime& operator =(const DateTime& DT);

      bool operator ==(const DateTime& DT);
      bool operator !=(const DateTime& DT);

      bool operator <(const DateTime& DT);
      bool operator >(const DateTime& DT);

      bool operator <=(const DateTime& DT);
      bool operator >=(const DateTime& DT);
};

#endif
